SystemOrganization addCategory: #'Zinc-Seaside'!

WAServerAdaptor subclass: #ZnZincServerAdaptor
	instanceVariableNames: 'server'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Seaside'!

!ZnZincServerAdaptor commentStamp: 'SvenVanCaekenberghe 11/10/2011 17:23' prior: 0!
I convert between Seaside and Zinc HTTP requests and responses.

Instance Variables:
	server	<ZnServer>
		
I can be started like any other Seaside adaptor:

	ZnZincServerAdaptor startOn: 8080
	
It is also possible to further customize me, like this:

	ZnZincServerAdaptor new
		port: 8080;
		serverClass: ZnManagingMultiThreadedServer;
		start!

!ZnZincServerAdaptor methodsFor: 'private' stamp: 'PaulDeBruicker 4/14/2012 09:25'!
basicStart
	self server
		delegate: self delegate;
		reader: [ :stream "Seaside wants to do its own text conversions" | ZnRequest readBinaryFrom: stream ];
		start! !

!ZnZincServerAdaptor methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 15:24'!
basicStop
	self server isNil
		ifFalse: [ self server stop ].
	self server: nil! !

!ZnZincServerAdaptor methodsFor: 'private' stamp: 'lr 7/29/2011 12:59'!
convertMultipart: entity
	| dictionary |
	dictionary := ZnMultiValueDictionary new.
	entity partsDo: [ :each |
		| value |
		value := each fileName isEmptyOrNil 
			ifTrue: [ 
				each hasEntity
					ifTrue: [ self codec url decode: each fieldValueString ]
					ifFalse: [ String new ] ] 
			ifFalse: [ self convertMultipartFileField: each ] .
		dictionary at: each fieldName put: value ].
	^ dictionary 
		! !

!ZnZincServerAdaptor methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/27/2011 20:25'!
convertMultipartFileField: part  
	| file |
	(file := WAFile new)
		fileName: part fileName;
		contentType: part contentType printString;
		contents: part contents.
	^ file! !

!ZnZincServerAdaptor methodsFor: 'defaults' stamp: 'pmm 9/8/2010 16:54'!
defaultCodec
	^ GRCodec forEncoding: 'utf-8'! !

!ZnZincServerAdaptor methodsFor: 'defaults' stamp: 'SvenVanCaekenberghe 10/18/2011 14:07'!
defaultZnServer
	^ ZnServer on: self port! !

!ZnZincServerAdaptor methodsFor: 'accessing' stamp: 'PaulDeBruicker 4/14/2012 09:25'!
delegate
	^ ZnSeasideServerAdaptorDelegate with: self! !

!ZnZincServerAdaptor methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 10/18/2011 14:08'!
isRunning
	^ server notNil and: [ server isRunning ]! !

!ZnZincServerAdaptor methodsFor: 'testing' stamp: 'pmm 9/8/2010 16:51'!
isStarting
	^ false ! !

!ZnZincServerAdaptor methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 10/18/2011 14:08'!
isStopped
	^ server isNil or: [ server isRunning not ]! !

!ZnZincServerAdaptor methodsFor: 'testing' stamp: 'pmm 9/8/2010 16:52'!
isStopping
	^ false ! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'pmm 9/9/2010 22:08'!
requestAddressFor: aZincRequest
	^ aZincRequest  headers
		at: ZnConstants remoteAddressHeader 
		ifAbsent: [ nil ]! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 4/7/2012 18:12'!
requestBodyFor: aZincRequest
	^ (aZincRequest method ~= #TRACE 
		and: [ aZincRequest hasEntity
			and: [ aZincRequest entity isEmpty not
				and: [ aZincRequest entity contentType ~= ZnMimeType applicationFormUrlEncoded
					and: [ aZincRequest entity contentType ~= ZnMimeType multiPartFormData ] ] ] ])
			ifTrue: [
				"Seaside wants to do its own text conversions"
				aZincRequest entity bytes asString ]
			ifFalse: [
				String new ]! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 2/27/2011 20:23'!
requestCookiesFor: aZincRequest
	| cookieString |
	cookieString := aZincRequest headers
		at: 'Cookie' ifAbsent: [ ^ #() ].
	^ WARequestCookie fromString: cookieString! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/3/2012 15:47'!
requestFieldsFor: aZincRequest
	| fields |
	fields := WARequestFields new.
	(aZincRequest method = #POST and: [ aZincRequest hasEntity ])
		ifTrue: [ 
			aZincRequest entity contentType = ZnMimeType applicationFormUrlEncoded
				ifTrue: [
					fields addAll: aZincRequest entity fields ].
			aZincRequest entity contentType = ZnMimeType multiPartFormData
				ifTrue: [
					fields addAll: (self convertMultipart: aZincRequest entity) ] ].
	^ fields
	! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 9/12/2010 10:58'!
requestHeadersFor: aZincRequest
	| fields |
	fields := Dictionary new.
	aZincRequest headersDo: [ :key :value |
		fields at: key asLowercase put: value ].
	^ fields! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'pmm 9/8/2010 17:15'!
requestMethodFor: aZincRequest
	^ aZincRequest method greaseString! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'PaulDeBruicker 04/14/2011 12:13'!
requestUrlFor: aZincRequest
	^ (WAUrl absolute: aZincRequest uri greaseString)
		decodedWith: self codec.! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'pmm 9/8/2010 17:19'!
requestVersionFor: aZincRequest
	^ WAHttpVersion fromString: aZincRequest requestLine version! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'pmm 4/27/2011 18:59'!
responseFrom: aRequestContext
	| partialHeaders cookies fullHeaders seasideResponse contents entity contentType |
	seasideResponse := aRequestContext response.
	partialHeaders := seasideResponse headers.
	cookies := seasideResponse cookies.
	fullHeaders := ZnHeaders defaultResponseHeaders.
	partialHeaders keysAndValuesDo: [ :key :value |
		fullHeaders at: key put: value ].
	cookies do: [ :each |
		fullHeaders at: 'Set-Cookie' add: each oldNetscapeString.
		fullHeaders at: 'Set-Cookie2' add: each rfc2965String ].
	contentType := seasideResponse contentType greaseString.
	contents := seasideResponse contents.
	entity := (ZnEntity bytes: contents) contentType: contentType; yourself.
	^ ZnResponse new
		statusLine: (ZnStatusLine code: seasideResponse status);
		headers: fullHeaders;
		entity: entity;
		yourself! !

!ZnZincServerAdaptor methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/18/2011 14:08'!
server
	"Return the ZnServer running me, create a default one if missing"
	
	^ server ifNil: [ server := self defaultZnServer ]! !

!ZnZincServerAdaptor methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 10/18/2011 15:06'!
server: znServer
	"I initialize the actual ZnServer instance that I will be using to znServer, 
	to prevent a fallback to #defaultZnServer"
	
	server := znServer! !

!ZnZincServerAdaptor methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 10/18/2011 15:06'!
serverClass: znServerClass
	"I initialize the actual ZnServer instance that I will be using, 
	by instanciating znServerClass using my port"
	
	self server: (znServerClass on: self port)! !

!ZnZincServerAdaptor methodsFor: 'private' stamp: 'lr 7/28/2011 18:19'!
shutDown
	self server isNil
		ifFalse: [ self server stop ]! !

!ZnZincServerAdaptor methodsFor: 'private' stamp: 'lr 7/28/2011 18:19'!
startUp
	self server isNil
		ifFalse: [ self server start ]! !

!ZnZincServerAdaptor methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/9/2010 20:38'!
statusString
	^ 'zinc on port ', self port greaseString, 
		( self isRunning ifTrue: [ ' [running]' ] ifFalse: [ ' [stopped]' ] )! !

ZnZincServerAdaptor subclass: #ZnZincStaticServerAdaptor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Seaside'!

!ZnZincStaticServerAdaptor commentStamp: 'PaulDeBruicker 5/11/2012 15:21' prior: 0!
This adaptor first checks to see if the Seaside response machinery creates a succesful response and if not checks to see if the request can be served by an instance of the ZnStaticFileServerDelegate 

Start it with something like: 


	ZnZincStaticServerAdaptor startOn: 8080 andServeFilesFrom: '/var/www/'.
	

If you want to set cache expiration headers for your static assets you can run this:

	ZnZincStaticServerAdaptor default useDefaultExpirationHeaders.

after you start the ZnZincStaticServerAdaptor.

If you want to set your own expiration headers you can create a Dictionary like the one in ZnStaticFileServerDelegate class>>#defaultMimeTypeExpirations and then run:

	ZnZincStaticServerAdaptor default useCustomExpirationHeaders: myExpriationHeadersDirectory.
	
where 'myExpirationHeadersDirectory' is structured similarly to the #defaultMimeTypeExpirations !

!ZnZincStaticServerAdaptor class methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 4/20/2012 16:21'!
startOn: aPortNumber andServeFilesFrom: aDirectoryPathString
	self startOn: aPortNumber.
	self default server delegate serveFilesFrom: aDirectoryPathString! !

!ZnZincStaticServerAdaptor methodsFor: 'accessing' stamp: 'PaulDeBruicker 4/19/2012 16:24'!
delegate
	^ ZnSeasideStaticServerAdaptorDelegate with: self! !

!ZnZincStaticServerAdaptor methodsFor: 'accessing' stamp: 'PaulDeBruicker 5/11/2012 14:45'!
useCustomExpirationHeaders: aDictionaryOfMimeTypesAndHeaders
	self server delegate staticDelegate mimeTypeMaxAges: aDictionaryOfMimeTypesAndHeaders! !

!ZnZincStaticServerAdaptor methodsFor: 'accessing' stamp: 'PaulDeBruicker 5/11/2012 14:44'!
useDefaultExpirationHeaders
	self server delegate staticDelegate mimeTypeMaxAges: ZnStaticFileServerDelegate defaultMimeTypeExpirations! !

Object subclass: #ZnSeasideServerAdaptorDelegate
	instanceVariableNames: 'adaptor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Seaside'!

!ZnSeasideServerAdaptorDelegate commentStamp: 'SvenVanCaekenberghe 9/30/2010 18:13' prior: 0!
I am ZnSeasideServerAdaptorDelegate.

I handle Zinc HTTP Server delegate's #handleRequest: using WAServerAdaptor>>#process:

Instance Variables:
	adaptor	<WAServerAdaptor>!

!ZnSeasideServerAdaptorDelegate class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/30/2010 17:14'!
with: aSeasideServerAdaptor
	^ self new
		adaptor: aSeasideServerAdaptor;
		yourself! !

!ZnSeasideServerAdaptorDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 17:13'!
adaptor
	^ adaptor ! !

!ZnSeasideServerAdaptorDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 17:12'!
adaptor: aSeasideServerAdaptor
	adaptor := aSeasideServerAdaptor ! !

!ZnSeasideServerAdaptorDelegate methodsFor: 'public' stamp: 'SvenVanCaekenberghe 9/30/2010 18:13'!
handleRequest: znRequest
	"I pass a Zinc HTTP Components ZnRequest to my Seaside Server Adaptor using #process:"
	
	^ self adaptor process: znRequest! !

ZnSeasideServerAdaptorDelegate subclass: #ZnSeasideStaticServerAdaptorDelegate
	instanceVariableNames: 'staticDelegate wwwRootDirectory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Seaside'!

!ZnSeasideStaticServerAdaptorDelegate commentStamp: 'PaulDeBruicker 4/20/2012 16:04' prior: 0!
This adaptor first checks to see if the Seaside response machinery creates a succesful response and if not checks to see if the request can be served by an instance of the ZnStaticFileServerDelegate 

Instance Variables:
	staticDelegate	aZnStaticFileServerDelegate
		wwwRootDirectory - The directory on your file system where the JS/CSS/JPEG etc. files you want to serve reside!

!ZnSeasideStaticServerAdaptorDelegate methodsFor: 'public' stamp: 'PaulDeBruicker 4/20/2012 16:13'!
handleRequest: znRequest
	"If the Seaside request processing apparatus returns an HTTP response with an error check the filesystem for a file that matches the uri of the request, if found send the file if not send an error."

	| response |
	response := self adaptor process: znRequest.
	response isError
		ifFalse: [ ^ response ].

	^ self staticDelegate handleRequest: znRequest! !

!ZnSeasideStaticServerAdaptorDelegate methodsFor: 'initialization' stamp: 'PaulDeBruicker 4/19/2012 16:28'!
initializeStaticDelegate
	^ ZnStaticFileServerDelegate new
		directory: self wwwRootDirectory;
		yourself! !

!ZnSeasideStaticServerAdaptorDelegate methodsFor: 'updating' stamp: 'PaulDeBruicker 4/20/2012 16:20'!
serveFilesFrom: aDirectoryPathString
	self wwwRootDirectory: (FileDirectory on: aDirectoryPathString )! !

!ZnSeasideStaticServerAdaptorDelegate methodsFor: 'accessing' stamp: 'PaulDeBruicker 4/19/2012 16:26'!
staticDelegate
	^ staticDelegate ifNil: [ staticDelegate := self initializeStaticDelegate ]! !

!ZnSeasideStaticServerAdaptorDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/20/2012 10:52'!
wwwRootDirectory
	"This is the root directory for your css/js/images etc...."

	^ wwwRootDirectory ifNil: [ wwwRootDirectory := FileDirectory on: '/var/www' ]! !

!ZnSeasideStaticServerAdaptorDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/20/2012 10:53'!
wwwRootDirectory: fileDirectory
	"This is the root directory for your css/js/images etc...."

	wwwRootDirectory := fileDirectory! !
